
#ifndef __htmdec__
#define __htmdec__

/** \brief Class for returning std::wstring over DLL-boundary
 *
 * When calling htmexp.dll functions from sim.dll both DLLs are not neccessarily
 * translated using the same Visual Studio version. When those differ, the
 * internal implementation of the string-class can also differ between the DLLs
 * so they can not simply be passed over the boundary, but have to be passed as
 * simple wchar_t*. Becuase that pointer can not be returned from a function
 * using c_str() because the string will be deleted after function ends and so
 * is the pointer and the calling function will get garbage, the passing back
 * has to be done using a callback function. This class is the implementation
 * for that callback functionality.
 */
class StdWStringReturn
{
    private:
        wstring rettext;
    public:
        static void setText(void* object, const wchar_t *text) { ((StdWStringReturn*)object)->rettext = text; };
        const wchar_t* getText(void) { return rettext.c_str(); };
};

string html_steuerzeichen2html(string str);
__declspec(dllimport) void html_steuerzeichen2html(const wchar_t *str, void* obj, void(*callback)(void*, const wchar_t*));
__declspec(dllimport) void html_ExportHtml(short gesamtdoku);
__declspec(dllimport) short html_openModul(char *filename, wstring titel, wstring mname, wstring info);
__declspec(dllimport) void html_closeModul(void);
__declspec(dllimport) void save_DispOverview(char *filename, short size_x, short size_y);
__declspec(dllimport) void html_disp2overview(short px, short py);
__declspec(dllimport) void html_writeMaskoverview(const char *text);
__declspec(dllimport) void html_writeMaskoverview(const wchar_t *text);

//*** Achtung: Funktionen die CView* oder CDC* verwenden knnen
//***			von der sim.dll nicht aufgerufen werden
#ifndef IMACS_01
__declspec(dllimport) void make_signaldoku(CView *viewhandle, short size_x, short size_y);
__declspec(dllimport) CDC *get_signaldoku(void);
__declspec(dllimport) void save_signaldoku(char *filename, short size_x, short size_y);

__declspec(dllimport) void make_smdoku(CView *viewhandle, short size_x, short size_y);
__declspec(dllimport) CDC *get_smdoku(void);
__declspec(dllimport) void html_jpg2Modul(char *filename, short x, short y);
__declspec(dllimport) void save_smdoku(char *filename, short size_x, short size_y);
__declspec(dllimport) void make_display(CView *viewhandle, short screenID, short, short, short, short, short, char *framepath, long backCol, long frontCol, short dis_orient, short bits_per_pixel, bool debugTargetDisplay);
__declspec(dllimport) void update_display(CDC *pdc, short screenID, short x, short y, bool forceUpdate);
__declspec(dllimport) void delete_display(void);
#endif

__declspec(dllimport) void html_setDokumentationstyp(short dt);
__declspec(dllimport) void html_smTableTitle(char *statname);
__declspec(dllimport) void html_smTableSection(char *sectionname, short nr);
__declspec(dllimport) void html_smTableEntry(char *entry1, char *entry2);
__declspec(dllimport) void html_smTableEnde(void);

__declspec(dllimport) void html_setDokuLang(short dl);
__declspec(dllimport) void copy_ctrbmp(unsigned char *bmp, long len);

__declspec(dllimport) void AddMainTitle(string maint);
__declspec(dllimport) void AddCustomer(const char *maint);
__declspec(dllimport) void AddLicense(const char *maint);
__declspec(dllimport) void AddVersion(char *vers, const char* commonvers);
__declspec(dllimport) void html_delete(void);
__declspec(dllimport) void html_AssToInhalt(wstring *txt, wstring *txt1); 
__declspec(dllimport) void html_openass(char *txt);
__declspec(dllimport) void beginSubstate(void);
__declspec(dllimport) void endSubstate(void);

__declspec(dllimport) void html_closeass(void);

__declspec(dllimport) void html_openpage(wchar_t *txt, wchar_t *txt1, wchar_t *mname, short InstanceNumber);

__declspec(dllimport) void html_closepage(char *imagestring);

__declspec(dllimport) void html_ErrorExit(short infonr);
__declspec(dllimport) void html_getMessText(short infonr, char *text);

__declspec(dllexport) double html_getTimeText(char *text);
__declspec(dllexport) void html_getDateText(char *text);

__declspec(dllexport) long html_getColor(short colornr);

void html_PageLink(char *txt, char *txt1);
__declspec(dllimport) void html_PageLink(wchar_t *txt, wchar_t *txt1);
__declspec(dllimport) void html_setVisLang(short visl);

__declspec(dllimport) void html_write(wstring txt); 
__declspec(dllimport) void html_writeass(const wchar_t *txt); 
__declspec(dllexport) void html_writeallgData(const wchar_t *txt);
void html_writeaction(char *txt);
void html_writeactionW(char *txt);
__declspec(dllimport) void html_writeactionW(const wchar_t *txt);
__declspec(dllimport) void html_writepage(const wchar_t *txt); 
__declspec(dllimport) void html_writeoverview(wchar_t *txt); 
void html_writeoverviewlink(char *txt, char *txt1);
__declspec(dllimport) void html_writeoverviewlink(const wchar_t *txt, const wchar_t *txt1);
void html_writeoverviewlistlink(char *txt, char *txt1);
__declspec(dllimport) void html_writeoverviewlistlink(const wchar_t *txt, const wchar_t *txt1);
__declspec(dllimport) void html_overviewup(void);
__declspec(dllimport) void html_overviewdown(void);
__declspec(dllimport) void dll_dis_str(short screenID, short x,short y,const wchar_t *txt,short len,unsigned char col,unsigned char colbk, long font);
__declspec(dllimport) unsigned long dll_dis_getStrDimension(short screenID, wchar_t *str, long font);
__declspec(dllimport) void dll_dis_bmp(short screenID,short x,short y, long bmpid);
__declspec(dllimport) void dll_dis_bmp_rot(short screenID, short x,short y, short rotX, short rotY, long bmpid, long rotkangle);
__declspec(dllimport) void dll_dis_save_area(short screenID, short x, short y, short width, short height, unsigned char areaID);
__declspec(dllimport) void dll_dis_area_init(short screenID, unsigned char anz);
__declspec(dllimport) void dll_dis_restore_area(short screenID, unsigned char areaID);
__declspec(dllimport) void dll_dis_getBmpSize(long bmpid, short *sizex, short *sizey);
__declspec(dllimport) void put_key_in_queue(short screenID,short key);
__declspec(dllimport) short get_actualKey(short screenID);
__declspec(dllimport) short get_key_from_queue(short screenID);
__declspec(dllimport) short del_key_queue(short screenID);
__declspec(dllimport) void release_key(short screenID);
__declspec(dllimport) short test_key_in_queue(short screenID);
__declspec(dllimport) void put_mouseclick_in_queue(short posx, short posy, short clicktype, short screenID);
__declspec(dllimport) short test_touch_in_queue(short screenID, short *x, short *y);
__declspec(dllimport) short get_touch_from_queue(short screenID, short *x, short *y);
__declspec(dllimport) short get_actualTouch(short screenID, short *x, short *y);
__declspec(dllimport) void del_touch_queue(short screenID);
__declspec(dllimport) short scan_touch(short screenID, short* posx, short* posy);
__declspec(dllimport) void dll_dis_switchPlayback(bool playback);
__declspec(dllimport) void dll_dis_clr(short screenID);
__declspec(dllimport) void dll_dis_clr(short screenID, short x, short y, short sizex, short sizey, short color);
__declspec(dllimport) void dll_dis_layer(short screenID, short layer);
__declspec(dllimport) void dll_dis_fill(short screenID, short x, short y, short sizex, short sizey, short fillmodus, short color, short pensize);
__declspec(dllimport) unsigned char dll_pauseSimulation(void);
__declspec(dllimport) void dll_dis_line(short screenID, short sx, short sy, short ex, short ey, short color, short pensize); 
__declspec(dllimport) void dll_dis_arc(short screenID, short x, short y, short radx, short rady, short startang, short endang, short color, short pensize);
__declspec(dllimport) void dll_dis_circ(short screenID, short x, short y, short radx, short rady, short color, short pensize);
__declspec(dllimport) void dll_cursor_clr(short screenID);
__declspec(dllimport) void dll_cursor_move(short screenID,short x, short y, short col, short colbk, short font);
__declspec(dllimport) void dll_cursor_set(short screenID,short x, short y, short col, short colbk, short cursorfont=8);
__declspec(dllimport) void dll_cursor_mode(short screenID,short mode);
__declspec(dllimport) void dll_dis_oversize(short stretchpos, short stretchsize);
__declspec(dllimport) void dll_dis_lockOversize(short lock);
__declspec(dllimport) void dll_set_pixel(short x, short y);
__declspec(dllimport) void dll_dis_initPixelOutput(short screenID, short x, short y, short sizex, short sizey);
__declspec(dllimport) long dll_dis_getRGBColor(short screenID, short col);
__declspec(dllimport) unsigned long dll_dis_getPixel(short screenID, short x, short y);
__declspec(dllimport) void dll_dis_setNextPixelPos(short screenID, short x, short y);
__declspec(dllimport) void dll_dis_incPrintPixels(short screenID, unsigned char r, unsigned char g, unsigned char b, short count);
__declspec(dllimport) void dll_dis_finishPixelOutput(short screenID);
__declspec(dllimport) short dll_get_disp_orient(void);
__declspec(dllimport) void get_env(char *def);
__declspec(dllimport) void html_writepar(void);
__declspec(dllimport) void html_writecom(void);
__declspec(dllimport) void html_commAddCANopen(unsigned short index, unsigned short subindex, const wchar_t* text);
__declspec(dllimport) void dll_doku_oversize(short onOff);

__declspec(dllimport) void dll_dis_connection_txt(short x,short y, const char *txt, short breite, short hoehe, short spaltenbreite, short zeilenhoehe);
__declspec(dllimport) void dll_checkWriteOutOfDisplay(short screenID);
__declspec(dllimport) void dll_resetWriteOutOfDisplay(short screenID);
__declspec(dllimport) void dll_dis_set_pixel(short screenID, short x, short s, short color);
__declspec(dllexport) void dll_dis_backcolor(short screenID,long bcolor);
__declspec(dllexport) void dll_dis_fgcolor(short screenID,long fgcolor);
__declspec(dllexport) void dll_dis_replaceColors(short screenID, short x, short y, short sizex, short sizey, long* fromCol, long* toCol, short anz);

#endif //__htmdec__