@echo off
setlocal ENABLEEXTENSIONS
setlocal EnableDelayedExpansion

rem $Id: devenv_sim.bat 2050 2009-09-18 12:40:08Z hstrauss $
rem
rem Copyright (c) IMACS GmbH Germany. All rights reserved.
rem
rem This script compiles the simulation project

if /I "%1"=="release" (
	set "radMONDir=..\..\common\radMON"
) else (
	set "radMONDir=..\..\common\radMONDebug"
)

if exist "..\..\common\release\sim.dll" del "..\..\common\release\sim.dll"
if exist "!radMONDir!\sim.dll" del "!radMONDir!\sim.dll"
if exist sim.dep del sim.dep

if not exist ..\sim\sim.sln goto NOSIMSLN

rem File format version of solution does NOT equal visual studio version
set VSVERS=
findstr /m /c:"Microsoft Visual Studio Solution File, Format Version 9.00" ..\sim\sim.sln >NUL 2>NUL
if !errorlevel!==0 (
	set VSVERS=8.0
	set VERSION=2005
)
findstr /m /c:"Microsoft Visual Studio Solution File, Format Version 10.00" ..\sim\sim.sln >NUL 2>NUL
if !errorlevel!==0 (
	set VSVERS=9.0
    set VERSION=2008
)
findstr /m /c:"Microsoft Visual Studio Solution File, Format Version 11.00" ..\sim\sim.sln >NUL 2>NUL
if !errorlevel!==0 (
	set VSVERS=10.0
    set VERSION=2010
)
findstr /m /c:"Microsoft Visual Studio Solution File, Format Version 12.00" ..\sim\sim.sln >NUL 2>NUL
if !errorlevel!==0 (
	findstr /m /c:"<PlatformToolset>v110" ..\sim\sim.vcxproj >NUL 2>NUL
	if !errorlevel!==0 (
		set VSVERS=11.0
        set VERSION=2012
	)
	findstr /m /c:"<PlatformToolset>v120" ..\sim\sim.vcxproj >NUL 2>NUL
	if !errorlevel!==0 (
		set VSVERS=12.0
        set VERSION=2013
	)
	findstr /m /c:"<PlatformToolset>v140" ..\sim\sim.vcxproj >NUL 2>NUL
	if !errorlevel!==0 (
		set VSVERS=14.0
        set VERSION=2015
	)
)
if "!VSVERS!"=="" goto VSUNKNOWN
echo Detected Visual Studio !VERSION! project

rem Delete old MSVC_BIN if any
set MSVC_BIN=
call "%OSDL_SYS%\ctr_lib\arg_batches\set_msvc_bin.bat" !VSVERS!

if "!MSVC_BIN!"=="" goto VSSLNNOTINSTALLED

call "%OSDL_SYS%\ctr_lib\arg_batches\cleanup_path.bat" >NUL 2>NUL
call "!MSVC_BIN!\vcvars32.bat" >NUL 2>NUL
set include="%include%;%OSDL_SYS%\CTR_LIB\INC;%OSDL_SYS%\Def"

if !VERSION LEQ 2008 (
	goto VCPROJ
) else (
	if !VSVER! GEQ 2012 (
	    call :CHECKXPSupport XPSupportPath
		set "CURDIR=%CD%"
		pushd.
		cd /D "!XPSupportPath!" >NUL 2>NUL
		if errorlevel 1 (
			echo error: Could not find XP support for Visual C++. Install XP support in Visual Studio. Refer to installation manual. > "!CURDIR!\sim.err"
			popd
			goto ERRORHANDLER
		)
		popd
	)
	goto VCXPROJ
)

:VCPROJ
set ERRORLEVEL=0
vcbuild /useenv sim.vcproj %1 /logfile:sim.err
if ERRORLEVEL 1 (
	set ERRORLEVEL=1
)
goto ERRORHANDLER

:VCXPROJ
set ERRORLEVEL=0
msbuild /p:Configuration=%1;Platform=Win32 /fl /flp:LogFile=sim.err;errorsonly;warningsonly /clp:Verbosity=n sim.sln
if ERRORLEVEL 1 (
	set ERRORLEVEL=1
)
goto ERRORHANDLER

:VSSLNNOTINSTALLED
set ERRORLEVEL=1
echo error: Could not find installation of Visual Studio !VSVERS! required for sim.sln > sim.err
goto ERRORHANDLER

:VSUNKNOWN
set ERRORLEVEL=1
echo error: Could not determine version of sim.sln solution. Corrupted file or unsupported version? > sim.err
goto ERRORHANDLER

:NOSIMSLN
set SupVersion=
if exist "%OSDL_SYS%\soft\Sim2005" set "SupVersion=!SupVersion! 8.0"
if exist "%OSDL_SYS%\soft\Sim2008" set "SupVersion=!SupVersion! 9.0"
if exist "%OSDL_SYS%\soft\Sim2010" set "SupVersion=!SupVersion! 10.0"
if exist "%OSDL_SYS%\soft\Sim2012" set "SupVersion=!SupVersion! 11.0"
if exist "%OSDL_SYS%\soft\Sim2013" set "SupVersion=!SupVersion! 12.0"
if exist "%OSDL_SYS%\soft\Sim2015" set "SupVersion=!SupVersion! 14.0"

for %%a in (!SupVersion!) do (
	for %%b in (VCExpress\ VisualStudio\) do (
		for %%c in (SOFTWARE\Microsoft\ SOFTWARE\Wow6432Node\Microsoft\) do (
			set KEY_NAME="HKEY_LOCAL_MACHINE\%%c%%b%%a\Setup\VC"
			for /F "usebackq tokens=2*" %%d in (`REG QUERY !KEY_NAME! /v ProductDir 2^>nul`) do (
				set SupFound=1
			)
		)
	)
)
if "!SupFound!"=="" goto VSNOTINSTALLED
set ERRORLEVEL=1
echo error: Unexpected error occurred: Sim.sln not found > sim.err
goto ERRORHANDLER

:VSNOTINSTALLED
set ERRORLEVEL=1
echo error: No supported Visual Studio Version detected > sim.err
goto ERRORHANDLER

:ERRORHANDLER
set OSDL_ERROR=!ERRORLEVEL!

"%OSDL_SYS%\tools\errorhandler" ..\develop\module.ref sim.err ..\develop\error2.txt 1 /ilLNK4224
IF EXIST %1\sim.dll (
	if /I "%1"=="release" (
		copy %1\sim.dll ..\..\common\release\sim.dll
	)
	copy %1\sim.dll !radMONDir!\sim.dll
)

cd ..\develop
exit /B !OSDL_ERROR! >NUL 2>NUL

:CHECKXPSupport
for %%a in (SOFTWARE\Microsoft\ SOFTWARE\Wow6432Node\Microsoft\) do (
	set KEY_NAME="HKEY_LOCAL_MACHINE\%%aMicrosoft SDKs\Windows\v7.1A\XPSupport"
	for /F "usebackq tokens=2*" %%b in (`REG QUERY !KEY_NAME! /v InstallationFolder 2^>nul`) do (
		set "%~1=%%c"
	)
)
goto :EOF