/****************************************************************
*   file ImacsProt.h                                            *
*                                                               *
*   IMACS communiction protocol									*
*                                                               *
* Copyright [C] by												*
*		IMACS GmbH												*
*       Mittelfeldstr. 25, D-70806 Kornwestheim					*
*       Tel.: 07154/8083-0, Fax.: 07154/8083-29					*
*       E-Mail: info@imacs-gmbh.de								*
****************************************************************/

#ifndef _IMACS_PROT_H_
#define _IMACS_PROT_H_

#if defined(__cplusplus)
extern "C" {
#endif

/*************************** DEFINES ****************************/
// Timeouts
#define		MODEM_HANGUP_TIME	60000L			// time after that Modem will hang up if no communication takes place, in ms (set to 1 min)
#define		PW_RESPONSE_TIME	60000L			// time waiting for password, then hangup, in ms (set to 1 min)
// Value for sending infinite burst packets (without pause)
#define		BURST_INFINITE		0xFF

// I/O-function prototypes
typedef short (*WRITERDNDATA)(unsigned char * buf, short nBytes);
typedef short (*READRDNDATA)(unsigned char * buf, short nBytes);
typedef short (*RDSTATUSRDNDATA)(void);
typedef short (*WRSTATUSRDNDATA)(void);
typedef void (*CLEARRDNDATA)(void);

// Locking-function prototype
typedef short (*FILETRANSMUTEX)(unsigned char lock);

// states of Communication state machine in protocolReceiver()
enum { 
	PR_WAIT_HANDSHAKE=0,
	PR_WAIT_DATA,
	PR_HEADER, 
	PR_DATA, 
	PR_CHECKSUM, 
	PR_MODEMTEST,	
	PR_MODEMANSWER, 
	PR_ERROR,
	PR_CALLBACK, 
	PR_BREAK, 
	PR_HANGUP, 
	PR_SET_MSN,
	PR_SET_PIN,
};

// Communication Medium
enum eCOMMedium
{
	eCOMMMed_direct = 0 ,				// direct Communication (RS232, USB etc)
	eCOMMMed_COMModem ,					// old style COM-Module, Modem (any type)
	eCOMMMed_SoModem  ,					// Socket-Module, anal. Modem (Xmodus AL3094S)
	eCOMMMed_SoISDN  ,					// Socket-Module, ISDN Modem (not yet tested)
	eCOMMMed_SoGSM						// Socket-Module, GSM Modem (Xmodus XM7000S)
};

// Burst mode states
enum eBurstMode
{
	BURST_INACTIVE = 0,
	BURST_REQUESTED,					// Burst mode is requested by target, but not activated from PC
	BURST_ACTIVE,						// Burst is activated from PC
};

// File transfer state
enum eFileTransState
{
	FT_STAT_READY = 0,
	FT_STAT_RUNNING,
	FT_STAT_FINISHED,
	FT_STAT_RESERVED,			// File transfer structure is reserved for a communication device
	FT_STAT_OPEN = 0x80,		// File is open (masked with other states)
};

// File transfer mutex requests
enum eFileTransMutex
{
	FT_MUTEX_UNLOCK = 0, // Unlock mutex
	FT_MUTEX_LOCK, // Lock mutex
};

struct sFileTransfer;
struct sRdnCircularDisplaybuffer;

// I/O-Interface for protocol
// for each communication channel such a structure must be created (e.g. by the main init)
// Protocol functions have such a structure as parameter.
typedef struct sRdnProtIface
{
	// data to be set externally at init
	WRITERDNDATA  writeRdnData;			// the write function of the interface. returns number of bytes written or Error Code (<0).
	READRDNDATA   readRdnData;			// the read function of the interface. Par: (). Returns: number of bytes written or Error Code (<0).
	RDSTATUSRDNDATA ReadStatusRdnData;	// read status request. returns number of available bytes for read.
	WRSTATUSRDNDATA WriteStatusRdnData;	// write status request. returns 0 if nothing to transmit, else a number !=0 
	CLEARRDNDATA  clearRdnData;			// clear receive buffer. discards all unread data in receive buffer
	
	unsigned long  timer_tick;			// Calling  period of function protocolReceiver() in ms

	unsigned char* inBuf;				// receive Buffer, must be RD_COM_RCV_BUFSIZE bytes long
	unsigned char* outBuf;				// transmit Buffer, must be RD_COM_SND_BUFSIZE bytes long

	// data to set externally at init or runtime
	unsigned char ComEnabled ;			// Communication enabled. If 0, no communication is done and Interface is available for other protocols
	unsigned char Medium;				// Communication Medium to be used, eCOMMedium
	unsigned char fEnableChksum;		// 1 : enable sending of checksum (see also function Setup_Empty_RDN_PROT_IFACE())

	// internal data of communication procedures
	unsigned char cmd1;					// Commandobyte 1 (byte 0 of protocol header)
	unsigned char cmd2;					// Commandobyte 2 (byte 1 of protocol header)
	unsigned short nData;				// number of bytes in Block (byte 2+3 of protocol header)
	
	unsigned char  state;				// sequencing of receive procedure
	unsigned char  old_state;

	unsigned long  timeout;				// general timeout counter, counts in ms
					
	#ifdef PROTOCOL
		#if PROTOCOL_VERSION==6
			unsigned char* g_BigBlkAdr;			// Base Adress in case of Big Block Transfer
		#endif
	#endif

	unsigned char  fParamsChanged;		// Flag is set, when parameters are changed, to notify visualization
	unsigned char  ParamVersionOk;		// The transferred Param Block has the correct Version number
	unsigned short ParamRecvd;			// Number of bytes already received in Parameter Block
	unsigned char  SystemVersionOk;		// The transferred SysParam Block has the correct Version number
	unsigned short SystemRecvd;			// Number of bytes already received in Sstem Parameter Block
	unsigned char  ProcVersionOk;		// The transferred Process Data Block has the correct Version number
	unsigned short ProcRecvd;			// Number of bytes already received in Process Data Block

#ifdef RD_LEGACY_COMMUNICATION
	unsigned char	OldProtvers409;		// switch to old (<= 4.09) protocol. 1=old protocol was detected and will be executed (till power off/on)
#endif

	struct sRdnProtIface* next;			// Pointer to next communication channel structure

	#if USE_BURST != 0
		unsigned long  bursttime;			// burst time counter, counts in ms
		unsigned long  burstrate ;			// repeat time for burst messages (in ms)
		unsigned char  burst_enable;		// burst mode enabled by host or application (eBurstMode)
		unsigned char  burst_send_amount;	// amount of burst packets to send, until pausing
	#endif

	#if _USE_SDCARD_ != 0
		struct sFileTransfer* pFileTrans;	// Pointer to file transfer structure
	#endif // _USE_SDCARD_

	#ifdef RD_USE_MODEM						//---------------MODEM Data --------------
		char szCallbackNum[30];				// Callback phone number
		unsigned long ulOnlineTime;			// timer to stop communication (by hangup) if no message was received for a certain time (1 min). counts ms
		long PwTime;						// timer for time to receive valid Password within limited time (30 sec). counts ms
		
		unsigned short iMdm;				// index reading Modem answer code in rpi->Inbuf
		char fModem;						// 1: Modem operating
		char fConnected;					// 1: Modem connected
		char nCallNum;						// number of chars in szCallbackNum
		char fModemError;					// 1 : Error from Modem (cleared when communcation is on the run)
		char fRequestSIM_MSN;				// 1 : Request SIM / Set MSN now (after Modemtest)
		char SignalStrength;				// Signal Strength, only usefull when GSM Modem is used, not valid during online
	
		#ifdef RD_USE_SMS
			unsigned char fSendSMS ;		// 1 : send SMS / 0 : handle standard protocol
		#endif									//---------------SMS --------------
		unsigned short nchar ;				// char index when parsing modem answer
		char parsingstate;					// sequencing for parsing modem answer
	#endif									//---------------MODEM --------------

	#if ((NO_EMB_HMI == 0) && (RD_NO_DISPLAYCOMM == 0) && (RD_DISPCMD_COMMUNICATION != 0))
		struct sRdnCircularDisplaybuffer* dispCmdMem;	// Pointer to circular display buffer used to store API commands and attributes
	#endif
}RDN_PROT_IFACE;

typedef struct sFileTransfer
{
	unsigned char* fileTransBuf;		// Pointer to buffer for data exchange with detached thread
	unsigned char fileTransReq;			// Request for action within detached thread (eFileTransRequest)
	unsigned char fileTransState;		// Status of detached thread (eFileTransState)
	unsigned short dataSizeInStatusOut;	// used to get size of transfer data to detached thread and status back
	FILETRANSMUTEX fileTransMutex;		// function for handling mutex (uses eFileTransMutex as parameter)
	RDN_PROT_IFACE* curCommThreadStruct;// Pointer to calling communication interface
}RD_FILE_TRANSFER;

/******************* FUNCTION DECLARATIONS *********************/

// this function must be called periodically in the timer - interrupt
extern void protocolReceiver(RDN_PROT_IFACE* rpi);

#ifdef RD_USE_MODEM
	#ifdef RD_USE_SMS
		// starting a SMS transmission.
		extern short start_sms(char* pszSMSC_No, char* pszSMS_No, char* pszSMS_Msg);
		// get sms state
	extern short get_sms_state(void);
	#endif
#endif
// setup Default data in rpi - data to be set externally must be set after call
extern void Setup_Empty_RDN_PROT_IFACE(RDN_PROT_IFACE* rpi);
// setup Default data in rft - data to be set externall must be set after call
extern void Setup_Empty_RD_FILE_TRANSFER(RD_FILE_TRANSFER* rft);
// setup empty circular display buffer for display communication
extern void Setup_Empty_Circular_Display_Buffer(struct sRdnCircularDisplaybuffer* dispMem);
// convert unsigned short from host byte order to network byte order
unsigned short RdHtons(unsigned short hostshort);
// convert unsigned short from network byte order to host byte order
unsigned short RdNtohs(unsigned short netshort);
// detach call to fileTransferProc from current thread
extern short detachCallFileTransfer(RD_FILE_TRANSFER* rft);

/******************* EXTERNALLY AVAILABLE VARIABLES ************************/

extern RDN_PROT_IFACE* g_psProtIfaceList; // Concatenated list of communication structures

#ifdef RD_USE_MODEM
extern RD_char* g_pszMSN ;				// MSN-String
#endif

#if defined(__cplusplus)
} // extern "C" {
#endif

#endif // _IMACS_PROT_H_
