/****************************************************************
*   file RDnComm.h						                        *
*                                                               *
*   radCASE communication										*
*                                                               *
* Copyright [C] by												*
*		IMACS GmbH												*
*       Mittelfeldstr. 25, D-70806 Kornwestheim					*
*       Tel.: 07154/8083-0, Fax.: 07154/8083-29					*
*       E-Mail: info@imacs-gmbh.de								*
****************************************************************/

#ifndef _RDNCOMM_H_
#define _RDNCOMM_H_

#include <cominterface.h>
#include <ImacsProt.h>

#if defined(__cplusplus)
extern "C" {
#endif

/*************************** DEFINES ****************************/


// Throw an error for 0 byte buffer sizes (even though the compiler will report this anyway the error message will hopefully help finding the error quicker)
#if ((NO_EMB_HMI == 0) && (RD_NO_DISPLAYCOMM == 0) && (RD_DISPCMD_COMMUNICATION != 0) && (RD_DISPCMD_BUFSIZE == 0))
  #error Value for RD_DISPCMD_BUFSIZE may not be 0. If you want to disable display communication set the Define RD_NO_DISPLAYCOMM to 1 instead.
#endif

#if ((NO_EMB_HMI == 0) && (RD_NO_DISPLAYCOMM == 0) && (RD_DISPCMD_COMMUNICATION != 0))
	#if RD_DISPCMD_MAXRETRY > 0xF
		#error Value of RD_DISPCMD_MAXRETRY may not exceed 15.
	#endif
#endif

#if RD_COM_SND_BUFSIZE < 32
#error Send buffer for communication is too small. Please set RD_COM_SND_BUFSIZE to a value of minimum 32 Bytes
#endif // RD_COM_SND_BUFSIZE < 32

#if RD_COM_RCV_BUFSIZE < 32
#error Receive buffer for communication is too small. Please set RD_COM_RCV_BUFSIZE to a value of minimum 32 Bytes
#endif // RD_COM_RCV_BUFSIZE < 32

// Protocol version
#define RD_PROT_VERS_MAJ			4
#define RD_PROT_VERS_MIN			16

// Data sending sizes
#define RD_COM_HEADERSIZE			4 // If changing this value keep alignment of data access in mind
#define RD_COM_CHKSUMSIZE			1
#define RD_COM_SENDDATASIZE			(RD_COM_SND_BUFSIZE-RD_COM_HEADERSIZE-RD_COM_CHKSUMSIZE)

// Defines for circular buffer structure used in display command communication
#define RD_CIRCBUFFER_ERRORFLAG			0x10 // Buffer overflow error occured
#define RD_CIRCBUFFER_FLUSHREMOTEBUFFER	0x20 // Redraw occured, means visualization has to empty buffers
#define RD_CIRCBUFFER_FLUSHLOCALBUFFER	0x40 // Partial redraw after buffer overflow, needs an empty buffer

/*************************** Typedefs ****************************/

#if ((NO_EMB_HMI == 0) && (RD_NO_DISPLAYCOMM == 0) && (RD_DISPCMD_COMMUNICATION != 0))
// Structure for a circular buffer
// Sender in this context means the surface-interpreter context sending data into the buffer
// Receiver means the Imacs-protocol handler receiving data from the buffer
typedef struct sRdnCircularDisplaybuffer
{
	unsigned char buffer[RD_DISPCMD_BUFSIZE];	// Circular buffer for storage of API commands and attributes
	unsigned int start;							// Start position of circular buffer data (sender may only read, receiver will change the value)
	unsigned int end;							// End position of circular buffer data (receiver may only read, sender will change the value)
	short curComDispLayer;						// Maximum layer to accept in circular buffer (seperate between background and foreground layer)
	unsigned char statusMutex;					// Used as mutex for access to status variable. Only sender is allowed to write to this
												// variable. The mechanism assumes a higher priority of the receiver, so the sender
												// will never interrupt the receiver and the receiver access can be assumed atomic.
												// The sender needs to set this variable before write access to the status variable, the receiver
												// needs to check (read) if it is allowed to write the status variable.
	unsigned char status;						// Status containing error flags, error count, redraw requests for Project monitor
}RDN_CIRCULAR_DISPLAYBUFFER;
#endif

// Function to process extended information in single element write, to be implemented project-dependent.
typedef void (*PROCESSEXTINFO)(char* info) ;

// Function to handle messages in an external (projectspecific) function
typedef void (*HANDLEMESSAGEEXT)(RDN_PROT_IFACE* rpi) ;

/******************* EXTERNALLY AVAILABLE VARIABLES ************************/

#ifdef RD_USE_MODEM
// Pointer to Modem password, must be set by application during init
extern RD_char * pPassModem ;
// Pointer to Equipment ID, must be set by application during init
extern RD_char * pEquipmentID ;
#endif

// project-dependent function to process extended info, to be set by application
extern PROCESSEXTINFO pProcessExtInfo ;
// Handler for application specific command
extern HANDLEMESSAGEEXT pHandleAppSpecCommands ;
// project-specific function to be called whenever an Element has been changed
extern HANDLEMESSAGEEXT pElementChangedCommands ;

/******************* FUNCTION DECLARATIONS *********************/
// initialize Burst 
extern void startBurstMode(RDN_PROT_IFACE* rpi);
// disable Burst
extern void endBurstMode(RDN_PROT_IFACE* rpi);
#if _USE_SDCARD_ != 0
void fileTransferProc(RD_FILE_TRANSFER* rft);
#endif
#if ((NO_EMB_HMI == 0) && (RD_NO_DISPLAYCOMM == 0) && (RD_DISPCMD_COMMUNICATION != 0))
extern void putIntoDisplayBuffer(void* data, unsigned int size);
extern void putDrawCommandIntoDisplayBuffer(unsigned char cmd);
#endif

#if defined(__cplusplus)
} // extern "C" {
#endif

#endif /* _RDNCOMM_H_ */
