//
// $Id: ctr_elem.h 8889 2016-03-11 15:48:34Z hstrauss $
//
// Copyright (c) IMACS GmbH Germany. All rights reserved.
//

#ifndef __CTR_ELEM_H__
#define __CTR_ELEM_H__

// *************************************************************************
// Global definitions
// *************************************************************************

#define MAXLEN_FORMAT_STR	30



//*** Subtext-Handling
#if SUBTEXT_ON != 0
	#define SUBTEXT_DECL	, short subTextNr
	#define SUBTEXT_VAR		, subTextNr
	#ifndef SUBTEXT_NORM
	 #define SUBTEXT_NORM	,0	
	#endif	
#else
	#define SUBTEXT_DECL	
	#define SUBTEXT_VAR		
	#define SUBTEXT_NORM		
#endif

//*** Einheiten-Konvertierung-Handling
#if RD_UNITCONVERT != 0
	#define UNITCONV_DECL	, short unitConv
	#define UNITCONV_VAR		, unitConv
	#ifndef UNITCONV_NORM
	  #define UNITCONV_NORM	,0	
	#endif	
#else
	#define UNITCONV_DECL
	#define UNITCONV_VAR
	#define UNITCONV_NORM
#endif


	
#if	(RD_SELMAXSIZE > 1)
	#define _defMask(v)			unsigned long	v[RD_SELMAXSIZE]	// definieren
	#define	_setMask(a)			do_setMask((a))						// setzt Maske auf 1
	#define	_fillMask(a)		do_fillMask((a))					// setzt alle Bits einer Maske auf 1
	#define _invMask(a)			do_invMask(a)						// a = ~a
	#define	_moveMask(a,b)		do_moveMask((a),(b))				// a = b
	#define	_putMask(a,b)		do_putMask((a),(b))					// a = b
	#define _andMask(a,b)		do_andMask((a),(b))					// a &= b
	#define _orMask(a,b)		do_orMask((a),(b))					// a |= b
	#define _shiftLeftMask(a,s)	do_shiftLeftMask((a),(s))			// a << s
	#define	_testMask(a,b)		do_testMask((a),(b))				// a & b
	#define	_isBiggerMask(a,b)	do_isBiggerMask((a),(b))			// vergleiche (a > b)	

	void  do_setMask(		unsigned long a[RD_SELMAXSIZE]);
	void  do_fillMask(		unsigned long a[RD_SELMAXSIZE]);
	void  do_invMask(		unsigned long a[RD_SELMAXSIZE]);
	void  do_moveMask(		unsigned long a[RD_SELMAXSIZE], unsigned long RD_MRAM *b[RD_SELMAXSIZE]);
	void  do_putMask(		unsigned long RD_MRAM *a[RD_SELMAXSIZE], unsigned long b[RD_SELMAXSIZE]);
	void  do_andMask(		unsigned long a[RD_SELMAXSIZE], unsigned long b[RD_SELMAXSIZE]);
	void  do_orMask(		unsigned long a[RD_SELMAXSIZE], unsigned long b[RD_SELMAXSIZE]);
	void  do_shiftLeftMask(	unsigned long a[RD_SELMAXSIZE], unsigned short shifts);
	short do_testMask(		unsigned long a[RD_SELMAXSIZE], unsigned long b[RD_SELMAXSIZE]);
	short do_isBiggerMask(	unsigned long a[RD_SELMAXSIZE], unsigned long b[RD_SELMAXSIZE]);
#else
	#define _defMask(v)			unsigned long	v
	#define	_setMask(a)			(a) = 1
	#define	_fillMask(a)		(a) = 0xffffffff
	#define	_invMask(a)			(a) = ~(a) 
	#define	_moveMask(a,b)		((a) = (*b))
	#define	_putMask(a,b)		((*a) = (b))
	#define	_andMask(a,b)		((a) &= (b))
	#define	_orMask(a,b)		((a) |= (b))
	#define _shiftLeftMask(a,s)	((a) <<= (s))
	#define	_testMask(a,b)		((a) & (b))
	#define	_isBiggerMask(a,b)	((a) > (b))		
#endif




// *************************************************************************
// E L E M E N T E 
// *************************************************************************

void	RD_MBIN *get_ptr(short index);
#ifndef GET_TXT_PTR
void	RD_MBIN *get_ptr2(unsigned long index);
#else
void	RD_MBIN *get_ptr2(unsigned long index, unsigned char accesstype); // bspw. in usercode.c als Funktion ueber GET_TXT_PTR aufgerufen
#endif

	//*** allgemein CElement ***********************
	struct S_Element
	{
		unsigned char				elType;			// Elementtype (ETYPE_xxxx) (Bit 0x80 = Access to current value through offset instead of pointer in case of module pointers)
		XPTR						def;			// Position der Binaerdaten (Offset innerhalb des OSDL-INI)
		struct S_Element RD_MROM	*next;
		#if RD_DMINELEMDESC != 0
			struct S_Modul RD_MROM	*mod;
		#endif
		#if USE_RDI_INDEX != 0
			unsigned short 			rdi_index;
		#endif
		#if RD_USEELEMARRAYS != 0
			unsigned short			arraysize;
		#endif
	} ;
	typedef struct S_Element CElement;
	

	struct S_ENum
	{
		CElement elel;
		void RD_MRAM *act;		// Zeiger auf den aktuellen Wert des ENum
		char actvalsize;		// Wertgroesse: 1=char, 2=short, 3=long, -1=unsigned char, -2=unsigned short, -4=unsigned long
	};
	
	struct S_EBin
	{
		CElement elel;
		void RD_MRAM *act;  // Zeiger auf den aktuellen Wert des EBin
		char actvalsize;    // Wertgroesse: 1=char, 2=short, 4=long, -1=unsigned char, -2=unsigned short, -4=unsigned long
		#if (RD_SELMAXSIZE > 1)
			unsigned long RD_MRAM *enable_sel[RD_SELMAXSIZE];  // Zeiger auf den enable selection wert (falls vorhanden)
		#else
			unsigned long RD_MRAM *enable_sel;
		#endif
	 };
	
	struct S_EStr
	{
		CElement elel;
		RD_char RD_MRAM *act;		// Zeiger auf den aktuellen Wert des EStr
	};
	
	struct S_ETim
	{
		CElement elel;
		void RD_MRAM *act; 
	};
	
	struct S_EDat
	{
		CElement elel;
		void RD_MRAM *act; 
	};

	typedef struct S_ENum CENum;
	typedef struct S_EStr CEStr;
	typedef struct S_EBin CEBin;
	typedef struct S_ETim CETim;
	typedef struct S_EDat CEDat;
	
	// Runtime dynamic elements
	struct S_ENumRD
	{
		CENum elem;
		DENum RD_MRAM* def;
	};

	struct S_EBinRD
	{
		CEBin elem;
		DEBin RD_MRAM* def;
	};

	struct S_EStrRD
	{
		CEStr elem;
		DEStr RD_MRAM* def;
	};

	struct S_ETimRD
	{
		CETim elem;
		DETim RD_MRAM* def;
	};

	struct S_EDatRD
	{
		CEDat elem;
		DEDat RD_MRAM* def;
	};

	typedef struct S_ENumRD CENumRD;
	typedef struct S_EBinRD CEBinRD;
	typedef struct S_EStrRD CEStrRD;
	typedef struct S_ETimRD CETimRD;
	typedef struct S_EDatRD CEDatRD;

	
	void debugElem(M_D_SCR CElement RD_MROM *); 
	long long2BurstTime(long t);
	long long2BurstDate(long d);
	
	long getUndefVal(void RD_MRAM *pnt, char size);
	void putUndefVal(void RD_MRAM *pnt, long val, char size);
	
	void CElement_getName(CElement RD_MROM *elem, RD_char RD_MRAM *buf);
	void CElement_getDesc(RD_CONST CElement RD_MROM *elem, RD_char RD_MRAM *buf SUBTEXT_DECL);
	void CElement_getInfo(RD_CONST CElement RD_MROM *elem,RD_char RD_MRAM *buf);
	void CElement_getActTxt(CElement RD_MROM *elem, short modeDisplay,short showinHEX, RD_char RD_MRAM *buf SUBTEXT_DECL);
	void CElement_getUnit(CElement RD_MROM *elem, short modeDisplay, RD_char RD_MRAM *buf SUBTEXT_DECL);
	void CElement_getOriginalUnit(CElement RD_MROM *elem, short modeDisplay, RD_char RD_MRAM *buf);
	void CElement_getRange(CElement RD_MROM *elem, unsigned char fUnit, RD_char RD_MRAM *buf);
	long CEStr_hash(RD_char* str);
#if RD_USEFLOAT != 0
	void CEBinGetValAsString(CElement RD_MROM *elem, long *pactval, RD_char * pstr, short * pformat, char * pfloatedit);
#else
	void CEBinGetValAsString(CElement RD_MROM *elem, long *pactval, RD_char * pstr, short * pformat);
#endif

	long CElement_getAct(RD_CONST CElement RD_MROM *elem);

	#if RD_USEFLOAT != 0
		float CElement_getActFlp(RD_CONST CElement RD_MROM *elem);
		float getUndefValFlp(void RD_MRAM *pnt, CElement RD_MROM *elem);
		void  CElement_setActFlp(RD_CONST CElement RD_MROM *elem, float wert);
		void  putUndefValFlp(void RD_MRAM *pnt, float val, CElement RD_MROM *elem);
	#endif

	#if RD_UNITCONVERT != 0
		long CElement_getActUc(RD_CONST CElement RD_MROM *elem);
	#endif
	long CElement_getOld(CElement RD_MROM *elem);
	
	void CElement_setAct(RD_CONST CElement RD_MROM *elem, long wert);
	void CEBin_setAct(CEBin RD_MROM *elem, long wert);
	void CETim_setAct(CETim RD_MROM *elem, long wert);
	void CEDat_setAct(CEDat	RD_MROM *elem, long wert);
	
	unsigned short CElement_isType(CElement RD_MROM *elem, short ass);
	void CElement_setup(CElement RD_MROM *elem);
	void CElement_edit(M_D_SCR CElement RD_MROM *elem);
	void CElement_editdauer(M_D_SCR CElement RD_MROM *elem, short wert);
	void CElement_edittoggle(CElement RD_MROM *elem);
	
	unsigned short CEBin_getFormat(CElement RD_MROM *elem);
	short CENum_getFormat(CElement RD_MROM *elem);
	short CENum_checkFormat(CElement RD_MROM *elem);
	
	void CElement_ValToTxt(CElement RD_MROM *elem,void RD_MRAM *p, short fUnit, short showinHEX, RD_char RD_MRAM *buf);
	void CElement_info_help(M_D_SCR  CElement RD_MROM *elem);
	void CElement_setScale(CElement RD_MROM *elem, long prozentwert);
	long CElement_getScale(CElement RD_MROM *elem, long wert);

	void CElement_setmaxRange(CElement RD_MROM *elem, long);
	void CElement_setminRange(CElement RD_MROM *elem, long);
	long CElement_getmaxRange(CElement RD_MROM *elem);
	long CElement_getminRange(CElement RD_MROM *elem);
	long CElement_getmaxRange1(CElement RD_MROM *elem UNITCONV_DECL);
	long CElement_getminRange1(CElement RD_MROM *elem UNITCONV_DECL);

	#if RD_USEFLOAT != 0
	void  CElement_setmaxRangeFlp(CElement RD_MROM *elem, float);
	void  CElement_setminRangeFlp(CElement RD_MROM *elem, float);
	float CElement_getmaxRangeFlp(CElement RD_MROM *elem);
	float CElement_getminRangeFlp(CElement RD_MROM *elem);
	#endif
	
	void CElement_getSetup(CElement RD_MROM *elem, short modeDisplay, RD_char RD_MRAM *buffer);
	
	unsigned short CElement_isChanged(CElement RD_MROM *elem);
	
	
	
	void CENum_getUnit(CElement RD_MROM *elem,short modeDisplay, RD_char RD_MRAM *buffer SUBTEXT_DECL);
	
	void CEBin_set_enablebit(CEBin RD_MROM *elem, short nr);
	void CEBin_reset_enablebit(CEBin RD_MROM *elem, short nr);
	void CEBin_put_enablemask(CEBin RD_MROM *elem, _defMask(mask));

	void CEBin_getSelStr(CElement RD_MROM *elem, unsigned char ind, RD_char RD_MRAM *buf SUBTEXT_DECL);
	void CEBin_getMultiSelStr(CElement RD_MROM *elem,long val,RD_char RD_MRAM *buf);
	void CEBin_getSelDesc(CElement RD_MROM *elem, unsigned char ind, RD_char RD_MRAM *buf);
	
	void enum2str(CElement RD_MROM *elem, long val, RD_char RD_MRAM *valstr, short showinHEX, short format);
	
	void CENum_ValToTxt(CElement RD_MROM *elem, void RD_MRAM *p, short fUnit, short showinHEX, RD_char RD_MRAM *buf);
	void CEBin_ValToTxt(CElement RD_MROM *elem, void RD_MRAM *p, RD_char RD_MRAM *buf SUBTEXT_DECL);
	void CEStr_ValToTxt(CElement RD_MROM *elem, void RD_MRAM *p, RD_char RD_MRAM *buf);
	void CETim_ValToTxt(CElement RD_MROM *elem, void RD_MRAM *p, RD_char RD_MRAM *buf);
	void CEDat_ValToTxt(CElement RD_MROM *elem, void RD_MRAM *p, RD_char RD_MRAM *buf, char mode);

	short c_toggleNum(RD_pchar, RD_size, short);
	short c_toggleStr(RD_pchar, short);
	#if RD_USEFLOAT != 0
		short c_toggle2(M_D_SCR unsigned short, char*, short);
	#endif
	
	short	c_edit_str(M_D_SCR short, short, RD_char RD_MRAM *, RD_size, RD_size, char RD_MRAM *, short);
	short	c_edit_num(M_D_SCR short, short, RD_char RD_MRAM *, short, short, short);
	void c_edit_num1(M_D_SCR short, RD_char RD_MRAM *);
	#if RD_USEFLOAT != 0
		short	c_edit_num2(M_D_SCR short, short, RD_char RD_MRAM *, short, short, short);
	#endif
	short	c_edit_dat(M_D_SCR short, short, RD_char RD_MRAM *, short, char mode);
	short	c_edit_tim(M_D_SCR short, short, RD_char RD_MRAM *, short, short mode);
	short	c_edit_sel(M_D_SCR short, short, unsigned char *, DEBin RD_MBIN *, short, _defMask(enablebit) );
	
#endif
