//
// $Id: ctr_persist.h 4688 2012-05-10 10:41:40Z wdaur $
//
/****************************************************************
*   file ctr_persist.h	                                        *
*                                                               *
* Copyright [C] by												*
*		IMACS GmbH												*
*       Mittelfeldstr. 25, D-70806 Kornwestheim					*
*       Tel.: 07154/8083-0, Fax.: 07154/8083-29					*
*       E-Mail: info@imacs-gmbh.de								*
****************************************************************/

#ifndef __CTR_PERSIST_H__
#define __CTR_PERSIST_H__

/*************************** DEFINES ****************************/

// set all undefined copy numbers to 1
#ifndef PARAM_NUM
#define PARAM_NUM		1		// Number of copies for Parameters
#endif
#ifndef SYSTEM_NUM
#define SYSTEM_NUM		1		// Number of copies for System Parameters
#endif
#ifndef KALI_NUM
#define KALI_NUM		1		// Number of copies for Calibration Data
#endif
#ifndef PROC_NUM
#define PROC_NUM		1		// Number of copies for Process Data
#endif
#ifndef REMA_NUM
#define REMA_NUM		1		// Number of copies for Remanent Data
#endif
#ifndef TOUCH_NUM
#define TOUCH_NUM		1		// Number of copies for Touch calibration Data
#endif

// define max. Size for Persistent storage queue
// this default size avoids any overflow except for those due to slow performance
#ifndef PERSQUEUE_SIZE
#define PERSQUEUE_SIZE	5+PARAM_NUM+SYSTEM_NUM+KALI_NUM+PROC_NUM+REMA_NUM
#endif

// Address Operators. Access via direct Address operator is sometimes required due to Visual Studio Bug
#define PARORG			&ParamData
#define SYSTEMORG       &SystemData
#define EEP_KALI_INIT   &KalibData

// Minimum distance between 2 PROC write cycles in seconds. 
// If PROC Varaibles change permanently, this Time must be set to 10 Minutes to achieve a lifetime of 20 years at 1 Mio permitted write cycles. This is the default.
// If no PROC Variable is likely to be changed permamently, this time might be shorter, 0 is a valid value. Negative values switch off the periodic update
#ifndef MIN_PROC_UPDATE_INTERVAL
#define MIN_PROC_UPDATE_INTERVAL 600
#endif

// Minimum distance between 2 REMA write cycles in seconds. 
// REMA variables are likely to be changed permanently, so this Time must be set to 10 Minutes to achieve a lifetime of 20 years at 1 Mio permitted write cycles. This is the default.
// Negative values switch off the periodic update, update must then be triggered by program means
#ifndef MIN_REMA_UPDATE_INTERVAL
#define MIN_REMA_UPDATE_INTERVAL 600
#endif

#ifndef NO_PERSISTENT_DATA			// otherwise no storage is required
// In case no storage is defined to store the persistent data, this section will generate a fatal error at compile time
#if ((!defined(MGMT_RAM)) && (!defined(MGMT_CUS)) && (EEPROM_ON == 0) && (FLASH_ON == 0))
#error "Persistent Data cannot be retained"
#endif

#if ((TOUCH_ON != 0) && (!defined(TOUCH_CUS)) && (!defined(TOUCH_OFF)) && (!defined(TOUCH_OFF_F)))
#error "Touch Calibration Data cannot be retained"
#endif

#if ((!defined(PARAM_CUS)) && ((EEPROM_ON == 0) || (!defined(PARAM_OFF))) && ((FLASH_ON == 0) || (!defined(PARAM_OFF_F))))
#error "Parameter cannot be retained"
#endif

#if (defined(PARAM_OFF) && defined(PARAM_OFF_F))
#error "Parameter retention ambigious"
#endif

#if ((!defined(SYSTEM_CUS)) && ((EEPROM_ON == 0) || (!defined(SYSTEM_OFF))) && ((FLASH_ON == 0) || (!defined(SYSTEM_OFF_F))))
#error "System Parameter cannot be retained"
#endif

#if (defined(SYSTEM_OFF) && defined(SYSTEM_OFF_F))
#error "System Parameter retention ambigious"
#endif


#if ((KALI_ON != 0) && ((!defined(KALI_CUS)) && (((EEPROM_ON == 0) || (!defined(KALI_OFF))) && ((FLASH_ON == 0) || (!defined(KALI_OFF_F))))))
#error "Calibration Data cannot be retained"
#endif

#if (defined(KALI_OFF) && defined(KALI_OFF_F))
#error "Calibration Data retention ambigious"
#endif

#if ((!defined(PROC_RAM)) && (!defined(PROC_CUS)) && ((EEPROM_ON == 0) || (!defined(PROC_OFF))) && ((FLASH_ON == 0) || (!defined(PROC_OFF_F))))
#error "Process Data cannot be retained"
#endif

#if (defined(PROC_OFF) && defined(PROC_OFF_F))
#error "Process Data retention ambigious"
#endif

#if ((!defined(REMA_RAM)) && (!defined(REMA_CUS)) && ((EEPROM_ON == 0) || (!defined(REMA_OFF))) && ((FLASH_ON == 0) || (!defined(REMA_OFF_F))))
#error "Remanent Data cannot be retained"
#endif

#if (defined(REMA_OFF) && defined(REMA_OFF_F))
#error "Process Data retention ambigious"
#endif

#if (((defined(PARPRESRV_OFF) || defined(PARPRESRV_OFF_F)) && (!(RD_VALUE_PRESERVE & 0x02))))
#error "Parameter preservation requires DP Setting 2"
#endif

#if (defined(PARPRESRV_OFF) && defined(PARPRESRV_OFF_F))
#error "Parameter preservation ambigious"
#endif

#if (((defined(SYSPRESRV_OFF) || defined(SYSPRESRV_OFF_F)) && (!(RD_VALUE_PRESERVE & 0x01))))
#error "System Parameter preservation requires DP Setting 1"
#endif

#if (defined(SYSPRESRV_OFF) && defined(SYSPRESRV_OFF_F))
#error "System Parameter preservation ambigious"
#endif

#if (((defined(PROCPRESRV_OFF) || defined(PROCPRESRV_OFF_F)) && (!(RD_VALUE_PRESERVE & 0x04))))
#error "Process Parameter preservation requires DP Setting 4"
#endif

#if (defined(PROCPRESRV_OFF) && defined(PROCPRESRV_OFF_F))
#error "Process Parameter preservation ambigious"
#endif

// Following error detection only in Visual Studio
#ifdef _MSC_VER
#if _MSC_VER >= 1600
#define PERSIST_ASSERT(exp, errtxt) static_assert(exp, errtxt)
#else // _MSC_VER < 1600
#define PERSIST_ASSERT(exp, errtxt) C_ASSERT(exp)
#endif // _MSC_VER >= 1600

// This macro returns the minimum of the two value which is bigger than the compareto
// value. If both values are smaller than that value the value itself is returned
#define PERSIST_BIGGERMIN(compareTo, val1, val2) (((val1>compareTo)&&(val1<val2))?val1:((val2>compareTo)?val2:compareTo))
// This macro compares all offsets and the EEPROM length to the offset passed to the macro. It returns
// the smallest bigger value
#define PERSIST_NEXTBIGGER(offset) (PERSIST_BIGGERMIN(offset, PERSIST_TOUCH_OFF, PERSIST_BIGGERMIN(offset, PERSIST_PARAM_OFF, PERSIST_BIGGERMIN(offset, PERSIST_SYSTEM_OFF, PERSIST_BIGGERMIN(offset, PERSIST_KALI_OFF, PERSIST_BIGGERMIN(offset, PERSIST_PROC_OFF, PERSIST_BIGGERMIN(offset, PERSIST_REMA_OFF, PERSIST_LEN)))))))
// The macro returns false if there are two offsets (or EEPROM_length) at the same offset passed into the 
// macro
#define PERSIST_ISZEROSIZE(offset) ((((offset == PERSIST_TOUCH_OFF)?1:0)+((offset == PERSIST_PARAM_OFF)?1:0)+((offset == PERSIST_SYSTEM_OFF)?1:0)+((offset == PERSIST_KALI_OFF)?1:0)+((offset == PERSIST_PROC_OFF)?1:0)+((offset == PERSIST_REMA_OFF)?1:0)+((offset == PERSIST_LEN)?1:0))>1)
// The macro returns the size of the memory starting at the given offset, if there
// are multiple offsets starting there the size is assumed 0
#define PERSIST_SIZESTARTINGAT(offset) (PERSIST_ISZEROSIZE(offset)?0:PERSIST_NEXTBIGGER(offset)-offset)

#if EEPROM_ON != 0
// Create temporary Defines for checking EEPROM memory assignment
#ifdef TOUCH_OFF
#define PERSIST_TOUCH_OFF		(TOUCH_OFF)
#else // TOUCH_OFF
#define PERSIST_TOUCH_OFF		(EEPROM_LEN)
#endif // TOUCH_OFF
#ifdef PARAM_OFF
#define PERSIST_PARAM_OFF		(PARAM_OFF)
#else // PARAM_OFF
#define PERSIST_PARAM_OFF		(EEPROM_LEN)
#endif // PARAM_OFF
#ifdef SYSTEM_OFF
#define PERSIST_SYSTEM_OFF		(SYSTEM_OFF)
#else // SYSTEM_OFF
#define PERSIST_SYSTEM_OFF		(EEPROM_LEN)
#endif // SYSTEM_OFF
#ifdef KALI_OFF
#define PERSIST_KALI_OFF		(KALI_OFF)
#else // KALI_OFF
#define PERSIST_KALI_OFF		(EEPROM_LEN)
#endif // KALI_OFF
#ifdef PROC_OFF
#define PERSIST_PROC_OFF		(PROC_OFF)
#else // PROC_OFF
#define PERSIST_PROC_OFF		(EEPROM_LEN)
#endif // PROC_OFF
#ifdef REMA_OFF
#define PERSIST_REMA_OFF		(REMA_OFF)
#else // REMA_OFF
#define PERSIST_REMA_OFF		(EEPROM_LEN)
#endif // REMA_OFF
#define PERSIST_LEN				(EEPROM_LEN)

// Check EEPROM size and assignment
#if (TOUCH_ON != 0) && defined(TOUCH_OFF)
PERSIST_ASSERT(TOUCH_OFF < EEPROM_LEN, "Offset of touch data bigger than EEPROM size");
// Currently check of touch is deactivated, because the TOUCHKALIB structure is not known here (defined in module.h)
//PERSIST_ASSERT(PERSIST_SIZESTARTINGAT(TOUCH_OFF) > sizeof(TOUCHKALIB)*TOUCH_NUM, "Rerserved memory too small for touch calibration data");
#endif // (TOUCH_ON != 0) && defined(TOUCH_OFF)
#ifdef PARAM_OFF
PERSIST_ASSERT((PARAM_OFF) < EEPROM_LEN, "Offset of parameter data bigger than EEPROM size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT((PARAM_OFF)) >= sizeof(PARAM)*PARAM_NUM, "Rerserved memory too small for parameter data");
#endif // PARAM_OFF
#ifdef SYSTEM_OFF
PERSIST_ASSERT((SYSTEM_OFF) < EEPROM_LEN, "Offset of system parameter data bigger than EEPROM size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT((SYSTEM_OFF)) >= sizeof(SYSTEM)*SYSTEM_NUM, "Rerserved memory too small for system parameter data");
#endif // SYSTEM_OFF
#if (KALI_ON != 0) && defined(KALI_OFF)
PERSIST_ASSERT((KALI_OFF) < EEPROM_LEN, "Offset of calibration data bigger than EEPROM size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT((KALI_OFF)) >= sizeof(EEP_Kali)*KALI_NUM, "Rerserved memory too small for calibration data");
#endif // (KALI_ON != 0) && defined(KALI_OFF)
#if !defined(PROC_RAM) && defined(PROC_OFF)
PERSIST_ASSERT((PROC_OFF) < EEPROM_LEN, "Offset of process data bigger than EEPROM size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT((PROC_OFF)) >= sizeof(RD_PROC)*PROC_NUM, "Rerserved memory too small for process data");
#endif // !defined(PROC_RAM) && defined(PROC_OFF)
#if !defined(REMA_RAM) && defined(REMA_OFF)
PERSIST_ASSERT((REMA_OFF) < EEPROM_LEN, "Offset of remanent data bigger than EEPROM size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT((REMA_OFF)) >= REMANENT_LEN*REMA_NUM, "Rerserved memory too small for remanent data");
#endif // !defined(REMA_RAM) && defined(REMA_OFF)

// Remove temporary Defines
#undef PERSIST_TOUCH_OFF
#undef PERSIST_PARAM_OFF
#undef PERSIST_SYSTEM_OFF
#undef PERSIST_KALI_OFF
#undef PERSIST_PROC_OFF
#undef PERSIST_REMA_OFF
#undef PERSIST_LEN
#endif // EEPROM_ON != 0

#if FLASH_ON != 0
// Create temporary Defines for checking Flash memory assignment
#ifdef TOUCH_OFF_F
#define PERSIST_TOUCH_OFF		TOUCH_OFF_F
#else // TOUCH_OFF_F
#define PERSIST_TOUCH_OFF		FLASH_LEN
#endif // TOUCH_OFF_F
#ifdef PARAM_OFF_F
#define PERSIST_PARAM_OFF		PARAM_OFF_F
#else // PARAM_OFF_F
#define PERSIST_PARAM_OFF		FLASH_LEN
#endif // PARAM_OFF_F
#ifdef SYSTEM_OFF_F
#define PERSIST_SYSTEM_OFF		SYSTEM_OFF_F
#else // SYSTEM_OFF_F
#define PERSIST_SYSTEM_OFF		FLASH_LEN
#endif // SYSTEM_OFF_F
#ifdef KALI_OFF_F
#define PERSIST_KALI_OFF		KALI_OFF_F
#else // KALI_OFF_F
#define PERSIST_KALI_OFF		FLASH_LEN
#endif // KALI_OFF_F
#ifdef PROC_OFF_F
#define PERSIST_PROC_OFF		PROC_OFF_F
#else // PROC_OFF_F
#define PERSIST_PROC_OFF		FLASH_LEN
#endif // PROC_OFF_F
#ifdef REMA_OFF_F
#define PERSIST_REMA_OFF		REMA_OFF_F
#else // REMA_OFF_F
#define PERSIST_REMA_OFF		FLASH_LEN
#endif // REMA_OFF_F
#define PERSIST_LEN				FLASH_LEN

// Check flash size
#if (TOUCH_ON != 0) && defined(TOUCH_OFF_F)
PERSIST_ASSERT(TOUCH_OFF_F < FLASH_LEN, "Offset of touch data bigger than flash size");
// Currently check of touch is deactivated, because the TOUCHKALIB structure is not known here (defined in module.h)
//PERSIST_ASSERT(PERSIST_SIZESTARTINGAT(TOUCH_OFF_F) > sizeof(TOUCHKALIB)*TOUCH_NUM, "Rerserved memory too small for touch calibration data");
#endif // (TOUCH_ON != 0) && defined(TOUCH_OFF_F)
#ifdef PARAM_OFF_F
PERSIST_ASSERT(PARAM_OFF_F < FLASH_LEN, "Offset of parameter data bigger than flash size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT(PARAM_OFF_F) >= sizeof(PARAM)*PARAM_NUM, "Rerserved memory too small for parameter data");
#endif // PARAM_OFF_F
#ifdef SYSTEM_OFF_F
PERSIST_ASSERT(SYSTEM_OFF_F < FLASH_LEN, "Offset of system parameter data bigger than flash size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT(SYSTEM_OFF_F) >= sizeof(SYSTEM)*SYSTEM_NUM, "Rerserved memory too small for system parameter data");
#endif // SYSTEM_OFF_F
#if (KALI_ON != 0) && defined(KALI_OFF_F)
PERSIST_ASSERT(KALI_OFF_F < FLASH_LEN, "Offset of calibration data bigger than flash size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT(KALI_OFF_F) >= sizeof(EEP_Kali)*KALI_NUM, "Rerserved memory too small for calibration data");
#endif // (KALI_ON != 0) && defined(KALI_OFF_F)
#if !defined(PROC_RAM) && defined(PROC_OFF_F)
PERSIST_ASSERT(PROC_OFF_F < FLASH_LEN, "Offset of process data bigger than flash size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT(PROC_OFF_F) >= sizeof(RD_PROC)*PROC_NUM, "Rerserved memory too small for process data");
#endif // !defined(PROC_RAM) && defined(PROC_OFF_F)
#if !defined(REMA_RAM) && defined(REMA_OFF_F)
PERSIST_ASSERT(REMA_OFF_F < FLASH_LEN, "Offset of remanent data bigger than flash size");
PERSIST_ASSERT(PERSIST_SIZESTARTINGAT(REMA_OFF_F) >= REMANENT_LEN*REMA_NUM, "Rerserved memory too small for remanent data");
#endif // !defined(REMA_RAM) && defined(REMA_OFF_F)

// Remove temporary Defines
#undef PERSIST_TOUCH_OFF
#undef PERSIST_PARAM_OFF
#undef PERSIST_SYSTEM_OFF
#undef PERSIST_KALI_OFF
#undef PERSIST_PROC_OFF
#undef PERSIST_REMA_OFF
#undef PERSIST_LEN
#endif // FLASH_ON != 0

#endif // _MSC_VER

#if (defined(PARPRESRV_OFF) || defined(PARPRESRV_OFF_F) || defined(SYSPRESRV_OFF) || defined(SYSPRESRV_OFF_F) || defined(PROCPRESRV_OFF) || defined(PROCPRESRV_OFF_F))
#define PRESERVE_SET			// this define is only used in ctr_persist.c and must not be set manually
#endif

// if preservation buffer sizes are not defined, they are defined here to allow approx. 250 Elements per class
#ifdef PRESERVE_SET
#ifndef PRS_DATABUFSIZE
#define PRS_DATABUFSIZE		1024		// size of buffer to store data during preservation
#endif
#ifndef PRS_TABLEBUFSIZE
#define PRS_TABLEBUFSIZE	1024		// size of buffer to store Preserve information (1 long per element)
#endif
#endif

#endif		// #ifndef NO_PERSISTENT_DATA


/******************* EXTERNAL VARIABLES ************************/

extern PARAM    ParamData ;
extern SYSTEM   SystemData ;
#if KALI_ON != 0
extern EEP_Kali KalibData ;
#endif
extern RD_PROC  RD_MRAM Process;


/******************* FUNCTION DECLARATIONS *********************/

// API functions, shall return 0 on success and a value <0 on fail
#if EEPROM_ON != 0
extern short Eep_readandwait (unsigned char *pDest, unsigned long Offset, long size);
extern short Eep_readorskip (unsigned char *pDest, unsigned long Offset, long size);
extern short Eep_writeandwait (unsigned char *pSrc, unsigned long Offset, long size);
extern short Eep_writeorskip (unsigned char *pSrc, unsigned long Offset, long size);

extern short Eep_State(void);	// 0 : available / != 0 : busy
#endif
#if FLASH_ON != 0
extern short Flash_readandwait (unsigned char *pDest, unsigned long Offset, long size);
extern short Flash_readorskip (unsigned char *pDest, unsigned long Offset, long size);
extern short Flash_writeandwait (unsigned char *pSrc, unsigned long Offset, long size);
extern short Flash_writeorskip (unsigned char *pSrc, unsigned long Offset, long size);

extern short Flash_State(void);	// 0 : available / != 0 : busy
#endif

// get persitent data from storage
extern short GetPersistentData(void);

// Functions for immediate update
// must be used with care to avoid deadlocks in different threads, API-dependant
extern short DoParRestore(void);
extern short DoSysRestore(void);
extern short DoKaliRestore(void);
extern short DoProcRestore(void);
extern short DoRemaRestore(void);
extern short DoTouchRestore(void);

// these are the the generated data tables for Data Preservation.
extern RD_CONST RD_IDX_PNT_PRSV_TAB ElemIdxPrsvPntTab[] ;
extern RD_CONST unsigned short RD_TabPreservePar[];
extern RD_CONST unsigned short RD_TabPreserveSys[];
extern RD_CONST unsigned short RD_TabPreserveProc[];

#endif // __CTR_PERSIST_H__
